@@ -7,7 +7,7 @@ use crate::{
 
 use digest::FixedOutput;
 use generic_array::GenericArray;
-use rand::{CryptoRng, RngCore};
+use rand_core::{CryptoRng, RngCore};
 
 /// Defines a combination of key exchange mechanism and a KDF, which together form a KEM
 pub trait Kem: Sized {
@@ -22,7 +22,7 @@ pub trait Kem: Sized {
     /// Requirements
     /// ============
     /// This keying material SHOULD have as many bits of entropy as the bit length of a secret key,
-    /// i.e., `8* Self::Kex::PrivateKey::size()`. For X25519 and P-256, this is 32 bytes of
+    /// i.e., `8 * Self::Kex::PrivateKey::size()`. For X25519 and P-256, this is 256 bits of
     /// entropy.
     fn derive_keypair(
         ikm: &[u8],
@@ -57,28 +57,28 @@ pub trait Kem: Sized {
 use Kem as KemTrait;
 
 #[cfg(feature = "x25519-dalek")]
-/// Represents DHKEM(Curve25519, HKDF-SHA256)
+/// Represents DHKEM(X25519, HKDF-SHA256)
 pub struct X25519HkdfSha256 {}
 
 #[cfg(feature = "x25519-dalek")]
-impl Kem for X25519HkdfSha256 {
+impl KemTrait for X25519HkdfSha256 {
     type Kex = crate::kex::X25519;
     type Kdf = crate::kdf::HkdfSha256;
 
-    // §7.1: DHKEM(X25519, HKDF-SHA256)
+    // draft11 §7.1: DHKEM(X25519, HKDF-SHA256)
     const KEM_ID: u16 = 0x0020;
 }
 
 #[cfg(feature = "p256")]
-/// Represents DHKEM(P256, HKDF-SHA256)
+/// Represents DHKEM(P-256, HKDF-SHA256)
 pub struct DhP256HkdfSha256 {}
 
 #[cfg(feature = "p256")]
-impl Kem for DhP256HkdfSha256 {
+impl KemTrait for DhP256HkdfSha256 {
     type Kex = crate::kex::DhP256;
     type Kdf = crate::kdf::HkdfSha256;
 
-    // §7.1: DHKEM(P-256, HKDF-SHA256)
+    // draft11 §7.1: DHKEM(P-256, HKDF-SHA256)
     const KEM_ID: u16 = 0x0010;
 }
 
@@ -115,25 +115,30 @@ impl<Kex: KeyExchange> Deserializable for EncappedKey<Kex> {
 pub(crate) type SharedSecret<Kem> =
     GenericArray<u8, <<<Kem as KemTrait>::Kdf as KdfTrait>::HashImpl as FixedOutput>::OutputSize>;
 
+// draft11 §4.1
 // def Encap(pkR):
 //   skE, pkE = GenerateKeyPair()
 //   dh = DH(skE, pkR)
-//   enc = Serialize(pkE)
+//   enc = SerializePublicKey(pkE)
 //
-//   pkRm = Serialize(pkR)
+//   pkRm = SerializePublicKey(pkR)
 //   kem_context = concat(enc, pkRm)
 //
+//   shared_secret = ExtractAndExpand(dh, kem_context)
+//   return shared_secret, enc
+//
 // def AuthEncap(pkR, skS):
 //   skE, pkE = GenerateKeyPair()
 //   dh = concat(DH(skE, pkR), DH(skS, pkR))
-//   enc = Serialize(pkE)
+//   enc = SerializePublicKey(pkE)
 //
-//   pkRm = Serialize(pkR)
-//   pkSm = Serialize(pk(skS))
+//   pkRm = SerializePublicKey(pkR)
+//   pkSm = SerializePublicKey(pk(skS))
 //   kem_context = concat(enc, pkRm, pkSm)
 //
 //   shared_secret = ExtractAndExpand(dh, kem_context)
 //   return shared_secret, enc
+
 /// Derives a shared secret that the owner of the recipient's pubkey can use to derive the same
 /// shared secret. If `sk_sender_id` is given, the sender's identity will be tied to the shared
 /// secret.
@@ -141,7 +146,7 @@ pub(crate) type SharedSecret<Kem> =
 /// Return Value
 /// ============
 /// Returns a shared secret and encapped key on success. If an error happened during key exchange,
-/// returns `Err(HpkeError::InvalidKeyExchange)`.
+/// returns `Err(HpkeError::EncapError)`.
 pub(crate) fn encap_with_eph<Kem: KemTrait>(
     pk_recip: &KemPubkey<Kem>,
     sender_id_keypair: Option<&(KemPrivkey<Kem>, KemPubkey<Kem>)>,
@@ -151,7 +156,7 @@ pub(crate) fn encap_with_eph<Kem: KemTrait>(
     let suite_id = kem_suite_id::<Kem>();
 
     // Compute the shared secret from the ephemeral inputs
-    let kex_res_eph = Kem::Kex::kex(&sk_eph, pk_recip)?;
+    let kex_res_eph = Kem::Kex::kex(&sk_eph, pk_recip).map_err(|_| HpkeError::EncapError)?;
 
     // The encapped key is the ephemeral pubkey
     let encapped_key = {
@@ -175,7 +180,8 @@ pub(crate) fn encap_with_eph<Kem: KemTrait>(
 
         // We want to do an authed encap. Do KEX between the sender identity secret key and the
         // recipient's pubkey
-        let kex_res_identity = Kem::Kex::kex(sk_sender_id, pk_recip)?;
+        let kex_res_identity =
+            Kem::Kex::kex(sk_sender_id, pk_recip).map_err(|_| HpkeError::EncapError)?;
 
         // concatted_secrets = kex_res_eph || kex_res_identity
         // Same no-alloc concat trick as above
@@ -191,7 +197,7 @@ pub(crate) fn encap_with_eph<Kem: KemTrait>(
         // HKDF-Expand call only errors if the output values are 255x the digest size of the hash
         // function. Since these values are fixed at compile time, we don't worry about it.
         let mut buf = <SharedSecret<Kem> as Default>::default();
-        extract_and_expand::<Kem>(&concatted_secrets, &suite_id, &kem_context, &mut buf)
+        extract_and_expand::<Kem>(concatted_secrets, &suite_id, kem_context, &mut buf)
             .expect("shared secret is way too big");
         buf
     } else {
@@ -210,7 +216,7 @@ pub(crate) fn encap_with_eph<Kem: KemTrait>(
         // digest size of the hash function. Since these values are fixed at compile time, we don't
         // worry about it.
         let mut buf = <SharedSecret<Kem> as Default>::default();
-        extract_and_expand::<Kem>(&kex_res_eph.to_bytes(), &suite_id, &kem_context, &mut buf)
+        extract_and_expand::<Kem>(&kex_res_eph.to_bytes(), &suite_id, kem_context, &mut buf)
             .expect("shared secret is way too big");
         buf
     };
@@ -226,7 +232,7 @@ pub(crate) fn encap_with_eph<Kem: KemTrait>(
 /// Return Value
 /// ============
 /// Returns a shared secret and encapped key on success. If an error happened during key exchange,
-/// returns `Err(HpkeError::InvalidKeyExchange)`.
+/// returns `Err(HpkeError::EncapError)`.
 pub(crate) fn encap<Kem: KemTrait, R>(
     pk_recip: &KemPubkey<Kem>,
     sender_id_keypair: Option<&(KemPrivkey<Kem>, KemPubkey<Kem>)>,
@@ -242,33 +248,35 @@ where
     encap_with_eph::<Kem>(pk_recip, sender_id_keypair, sk_eph)
 }
 
+// draft11 §4.1
 // def Decap(enc, skR):
-//   pkE = Deserialize(enc)
+//   pkE = DeserializePublicKey(enc)
 //   dh = DH(skR, pkE)
 //
-//   pkRm = Serialize(pk(skR))
+//   pkRm = SerializePublicKey(pk(skR))
 //   kem_context = concat(enc, pkRm)
 //
 //   shared_secret = ExtractAndExpand(dh, kem_context)
 //   return shared_secret
 //
 // def AuthDecap(enc, skR, pkS):
-//   pkE = Deserialize(enc)
+//   pkE = DeserializePublicKey(enc)
 //   dh = concat(DH(skR, pkE), DH(skR, pkS))
 //
-//   pkRm = Serialize(pk(skR))
-//   pkSm = Serialize(pkS)
+//   pkRm = SerializePublicKey(pk(skR))
+//   pkSm = SerializePublicKey(pkS)
 //   kem_context = concat(enc, pkRm, pkSm)
 //
 //   shared_secret = ExtractAndExpand(dh, kem_context)
 //   return shared_secret
+
 /// Derives a shared secret given the encapsulated key and the recipients secret key. If
 /// `pk_sender_id` is given, the sender's identity will be tied to the shared secret.
 ///
 /// Return Value
 /// ============
 /// Returns a shared secret on success. If an error happened during key exchange, returns
-/// `Err(HpkeError::InvalidKeyExchange)`.
+/// `Err(HpkeError::DecapError)`.
 pub(crate) fn decap<Kem: KemTrait>(
     sk_recip: &KemPrivkey<Kem>,
     pk_sender_id: Option<&KemPubkey<Kem>>,
@@ -278,7 +286,8 @@ pub(crate) fn decap<Kem: KemTrait>(
     let suite_id = kem_suite_id::<Kem>();
 
     // Compute the shared secret from the ephemeral inputs
-    let kex_res_eph = Kem::Kex::kex(&sk_recip, &encapped_key.0)?;
+    let kex_res_eph =
+        Kem::Kex::kex(sk_recip, &encapped_key.0).map_err(|_| HpkeError::DecapError)?;
 
     // Compute the sender's pubkey from their privkey
     let pk_recip = Kem::Kex::sk_to_pk(sk_recip);
@@ -299,7 +308,8 @@ pub(crate) fn decap<Kem: KemTrait>(
 
         // We want to do an authed encap. Do KEX between the sender identity secret key and the
         // recipient's pubkey
-        let kex_res_identity = Kem::Kex::kex(sk_recip, pk_sender_id)?;
+        let kex_res_identity =
+            Kem::Kex::kex(sk_recip, pk_sender_id).map_err(|_| HpkeError::DecapError)?;
 
         // concatted_secrets = kex_res_eph || kex_res_identity
         // Same no-alloc concat trick as above
@@ -316,9 +326,9 @@ pub(crate) fn decap<Kem: KemTrait>(
         // function. Since these values are fixed at compile time, we don't worry about it.
         let mut shared_secret = <SharedSecret<Kem> as Default>::default();
         extract_and_expand::<Kem>(
-            &concatted_secrets,
+            concatted_secrets,
             &suite_id,
-            &kem_context,
+            kem_context,
             &mut shared_secret,
         )
         .expect("shared secret is way too big");
@@ -342,7 +352,7 @@ pub(crate) fn decap<Kem: KemTrait>(
         extract_and_expand::<Kem>(
             &kex_res_eph.to_bytes(),
             &suite_id,
-            &kem_context,
+            kem_context,
             &mut shared_secret,
         )
         .expect("shared secret is way too big");
